@@ -29,6 +29,8 @@
 
 #include "fs-rawudp-component.h"
 
+#include "fs-rawudp-marshal.h"
+
 #include <stun/usages/bind.h>
 #include <stun/stunagent.h>
 #include <stun/usages/timer.h>
@@ -43,8 +45,6 @@
 #include <libgupnp-igd/gupnp-simple-igd-thread.h>
 #endif
 
-#include <gio/gio.h>
-
 #include <string.h>
 #include <sys/types.h>
 
@@ -52,6 +52,15 @@
 # include <unistd.h>
 #endif
 
+#ifdef G_OS_WIN32
+# include <winsock2.h>
+#else /*G_OS_WIN32*/
+# include <netdb.h>
+# include <sys/socket.h>
+# include <netinet/in.h>
+# include <arpa/inet.h>
+#endif /*G_OS_WIN32*/
+
 #define GST_CAT_DEFAULT fs_rawudp_transmitter_debug
 
 #define DEFAULT_UPNP_MAPPING_TIMEOUT (600)
@@ -447,7 +456,10 @@ fs_rawudp_component_class_init (FsRawUdpComponentClass *klass)
     ("new-active-candidate-pair",
         G_TYPE_FROM_CLASS (klass),
         G_SIGNAL_RUN_LAST,
-        0, NULL, NULL, NULL,
+        0,
+        NULL,
+        NULL,
+        _fs_rawudp_marshal_VOID__BOXED_BOXED,
         G_TYPE_NONE, 2, FS_TYPE_CANDIDATE, FS_TYPE_CANDIDATE);
 
  /**
@@ -464,7 +476,10 @@ fs_rawudp_component_class_init (FsRawUdpComponentClass *klass)
     ("known-source-packet-received",
       G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST,
-      0, NULL, NULL, NULL,
+      0,
+      NULL,
+      NULL,
+      _fs_rawudp_marshal_VOID__UINT_POINTER,
       G_TYPE_NONE, 2, G_TYPE_UINT, G_TYPE_POINTER);
 
   /**
@@ -479,7 +494,10 @@ fs_rawudp_component_class_init (FsRawUdpComponentClass *klass)
   signals[ERROR_SIGNAL] = g_signal_new ("error",
       G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST,
-      0, NULL, NULL, NULL,
+      0,
+      NULL,
+      NULL,
+      _fs_rawudp_marshal_VOID__ENUM_STRING,
       G_TYPE_NONE, 2, FS_TYPE_ERROR, G_TYPE_STRING);
 
 
@@ -643,6 +661,11 @@ fs_rawudp_component_stop (FsRawUdpComponent *self)
         fs_rawudp_transmitter_udpport_remove_dest (udpport,
             self->priv->remote_candidate->ip,
             self->priv->remote_candidate->port);
+      else
+        fs_rawudp_transmitter_udpport_remove_recvonly_dest (udpport,
+            self->priv->remote_candidate->ip,
+            self->priv->remote_candidate->port);
+
 
       fs_rawudp_transmitter_udpport_remove_known_address (udpport,
           self->priv->remote_address, remote_is_unique_cb, self);
@@ -756,6 +779,8 @@ fs_rawudp_component_set_property (GObject *object,
         {
           if (sending)
           {
+            fs_rawudp_transmitter_udpport_remove_recvonly_dest (
+                self->priv->udpport, candidate->ip, candidate->port);
             fs_rawudp_transmitter_udpport_add_dest (self->priv->udpport,
                 candidate->ip, candidate->port);
           }
@@ -763,6 +788,8 @@ fs_rawudp_component_set_property (GObject *object,
           {
             fs_rawudp_transmitter_udpport_remove_dest (self->priv->udpport,
                 candidate->ip, candidate->port);
+            fs_rawudp_transmitter_udpport_add_recvonly_dest (
+                self->priv->udpport, candidate->ip, candidate->port);
           }
         }
         if (candidate)
@@ -910,6 +937,9 @@ fs_rawudp_component_set_remote_candidate (FsRawUdpComponent *self,
 {
   FsCandidate *old_candidate = NULL;
   gboolean sending;
+  struct addrinfo hints = {0};
+  struct addrinfo *res = NULL;
+  int rv;
   GInetAddress *addr;
 
   if (candidate->component_id != self->priv->component)
@@ -921,11 +951,12 @@ fs_rawudp_component_set_remote_candidate (FsRawUdpComponent *self,
     return FALSE;
   }
 
-  addr = g_inet_address_new_from_string (candidate->ip);
-  if (addr == NULL)
+  hints.ai_flags = AI_NUMERICHOST;
+  rv = getaddrinfo (candidate->ip, NULL, &hints, &res);
+  if (rv != 0)
   {
     g_set_error (error, FS_ERROR, FS_ERROR_INVALID_ARGUMENTS,
-        "Invalid address passed: %s", candidate->ip);
+        "Invalid address passed: %s", gai_strerror (rv));
     return FALSE;
   }
 
@@ -936,7 +967,6 @@ fs_rawudp_component_set_remote_candidate (FsRawUdpComponent *self,
     g_set_error (error, FS_ERROR, FS_ERROR_INVALID_ARGUMENTS,
         "Can't call set_remote_candidate after the thread has been stopped");
     FS_RAWUDP_COMPONENT_UNLOCK (self);
-    g_object_unref (addr);
     return FALSE;
   }
 
@@ -950,6 +980,25 @@ fs_rawudp_component_set_remote_candidate (FsRawUdpComponent *self,
 
   g_clear_object (&self->priv->remote_address);
 
+  switch (res->ai_family)
+  {
+    case AF_INET:
+      addr = g_inet_address_new_from_bytes (
+        (guint8*) &(((struct sockaddr_in *)res->ai_addr)->sin_addr.s_addr),
+        G_SOCKET_FAMILY_IPV4);
+      break;
+    case AF_INET6:
+      addr = g_inet_address_new_from_bytes (
+        (guint8*) &(((struct sockaddr_in6 *)res->ai_addr)->sin6_addr.s6_addr),
+        G_SOCKET_FAMILY_IPV6);
+      break;
+    default:
+      g_set_error (error, FS_ERROR, FS_ERROR_INTERNAL,
+        "Unknown address family");
+      return FALSE;
+  }
+
+
   self->priv->remote_address = g_inet_socket_address_new (addr,
       candidate->port);
   g_object_unref (addr);
@@ -960,9 +1009,14 @@ fs_rawudp_component_set_remote_candidate (FsRawUdpComponent *self,
 
   FS_RAWUDP_COMPONENT_UNLOCK (self);
 
+  freeaddrinfo (res);
+
   if (sending)
     fs_rawudp_transmitter_udpport_add_dest (self->priv->udpport,
         candidate->ip, candidate->port);
+  else
+    fs_rawudp_transmitter_udpport_add_recvonly_dest (self->priv->udpport,
+        candidate->ip, candidate->port);
 
   if (old_candidate)
   {
@@ -970,6 +1024,10 @@ fs_rawudp_component_set_remote_candidate (FsRawUdpComponent *self,
       fs_rawudp_transmitter_udpport_remove_dest (self->priv->udpport,
           old_candidate->ip,
           old_candidate->port);
+    else
+      fs_rawudp_transmitter_udpport_remove_recvonly_dest (self->priv->udpport,
+          candidate->ip,
+          candidate->port);
     fs_candidate_destroy (old_candidate);
   }
 
@@ -1339,7 +1397,7 @@ stun_recv_cb (GstPad *pad, GstPadProbeInfo *info, gpointer user_data)
   socklen_t addr_len = sizeof(addr);
   struct sockaddr_storage alt_addr;
   socklen_t alt_addr_len = sizeof(alt_addr);
-  gchar addr_str[NICE_ADDRESS_STRING_LEN + 1];
+  gchar addr_str[NI_MAXHOST];
   NiceAddress niceaddr;
   GstMapInfo map;
 
